#####################################################
# script to explore different intervention scenarios, including school closure
#####################################################


#####################################################
# source function that runs simulation
source('simulate_idcontrolmultigroup_ode.R')

#####################################################
## set initial conditions and parameters
#####################################################

#####################################################
##Initial Conditions
Ntot = 1e6
## assume 1 infected in each age group, no recovered, no dead
Ic = 1; Ia = 1; Ie = 1
#population with 20% children, 25% elderly, rest adults
Sc = Ntot*0.2 - Ic; Se = Ntot*0.25 - Ie; Sa = Ntot- Sc - Se - Ic - Ia - Ie

#####################################################
#####################################################
##Parameters
#####################################################
#####################################################
## Duration of infectious period. 
#####################################################
gc = 0.1; ga = 0.1; ge = 0.1
#####################################################
## Proportion dying  
#####################################################
mc = 0.001; ma = 0.01; me = 0.1
#####################################################
## transmission parameters  
#####################################################
## simplified version of POLYMOD/BBC/Steele 2016 Epidemics
bcc = 1; baa = 1; bee = 0.75
k1 = 0.5 #relative mixing between adults and children
k2 = 0.5 #relative mixing between adults and elderly
k3  = 0.25 #relative mixing between elderly and children
bca = k1*bcc; bac = bca
bae = k2*baa; bea = bae;
bec = k3*bcc; bce = bec; 

scale = 2.9e-7 #scaling factor, set to get an R0 of 2

bcc = scale * bcc; bca = scale * bca; bce = scale * bce
bac = scale * bac; baa = scale * baa; bae = scale * bae 
bec = scale * bec; bea = scale * bea; bee = scale * bee

## compute next generation matrix to get R0, use that to adjust 'scale' above until R0=2
T = matrix(data = c(bcc*Sc, bac*Sc, bec*Sc, bca*Sa, baa*Sa, bea*Sa, bce*Se, bae*Se, bee*Se), nrow = 3)
Sig = matrix(data = c(gc, 0, 0, 0, ga, 0, 0, 0, ge), nrow = 3)
EV = eigen(T %*% MASS::ginv(Sig))
R0 = max(EV$values)
print(R0)

#####################################################
#no waning immunity
#####################################################
wc = 0; wa = 0;  we = 0

#####################################################
## Intervention  
## not set here set below
#####################################################

#####################################################
#Other settings
#####################################################
tmax = 600 #total simulation time, all in days

#####################################################
# simulation runs for different control strategies
#####################################################

scenariovec = 1:5 #run over different scenarios, right now only each target group

#save results for each run
n_tot = length(scenariovec)
allres = data.frame(Scenario = rep(0,n_tot), Strength = rep(0,n_tot), Stot = rep(0,n_tot), Itot = rep(0,n_tot), Dtot = rep(0,n_tot))

allres = cbind(allres,matrix(0,nrow = n_tot,ncol = 12))

for (n1 in 1:length(scenariovec))
{
  
  if (n1 == 1) {f1=0;   f2=0;    f3=0} #no intervention
  if (n1 == 2) {f1=0.9; f2=0.9;  f3=1; } #strong intervention
  if (n1 == 3) {f1=1; f2=0.339;  f3=0; } #children intervention
  if (n1 == 4) {f1=0; f2=0.475;  f3=0; } #adult intervention
  if (n1 == 5) {f1=0; f2=0.318;  f3=1; } #elderly intervention
  
  #compute R0 for each intervention
  #just for checking to ensure R-effective is similar across interventions
  T =  matrix(data = c((1-f1)*bcc*Sc, (1-f1)*bac*Sc, (1-f1)*bec*Sc, (1-f2)*bca*Sa, (1-f2)*baa*Sa, (1-f2)*bea*Sa, (1-f3)*bce*Se, (1-f3)*bae*Se, (1-f3)*bee*Se), nrow = 3)
  Sig = matrix(data = c(gc, 0, 0, 0, ga, 0, 0, 0, ge), nrow = 3)
  EV = eigen(T %*% MASS::ginv(Sig))
  #print(EV)
  Reff = max(EV$values)
  print(Reff)
    
  f1_start = 100; f1_end = f1_start+120;
  f2_start = f1_start; f2_end = f1_end;
  f3_start = f1_start; f3_end = f1_end;
  
  ## run simulation
  result = simulate_idcontrolmultigroup_ode(Sc = Sc, Ic = Ic, Sa = Sa, Ia = Ia, Se = Se, Ie = Ie, bcc = bcc, bca = bca, bce = bce, bac = bac, baa = baa, bae = bae, bec = bec, bea = bea , bee = bee, gc = gc, ga = ga, ge = ge, wc = wc, wa = wa, we = we, mc = mc, ma = ma, me = me, f1  = f1, f1_start = f1_start, f1_end = f1_end, f2 = f2, f2_start = f2_start, f2_end = f2_end, f3 = f3, f3_start = f3_start, f3_end = f3_end,  tmax = tmax)
  
  #keep all time series for plotting
  #really bad way of growing array, but ok for here
  if (n1 == 1)
  {
    allts = result$ts  
    allts$Scenario = scenariovec[n1]
  }
  else
  {
    nowres = result$ts
    nowres$Scenario = scenariovec[n1]
    allts = rbind(allts,nowres)
  }
} #end scenario loop  

#save simulation results, will be loaded and processed by script that generates plots
filename = 'results.rds'
alldat = list()
alldat$allts=allts
alldat$f_start = f1_start
alldat$f_end = f1_end
alldat$Ntot = Ntot
alldat$R0 = R0
saveRDS(alldat, file = filename)

